#ifndef __CPHIDGETADVANCEDSERVO
#define __CPHIDGETADVANCEDSERVO
#include "cphidget.h"

/** \defgroup phidadvservo Phidget Advanced Servo 
 * \ingroup phidgets
 * @{
 */

DPHANDLE(AdvancedServo)
CHDRSTANDARD(AdvancedServo)

CHDRGET(AdvancedServo,MotorCount,int *count)

CHDRGETINDEX(AdvancedServo,Acceleration,double *acceleration)
CHDRSETINDEX(AdvancedServo,Acceleration,double acceleration)
CHDRGETINDEX(AdvancedServo,AccelerationMax,double *max)
CHDRGETINDEX(AdvancedServo,AccelerationMin,double *min)

//set velocity
CHDRGETINDEX(AdvancedServo,VelocityLimit,double *imit)
CHDRSETINDEX(AdvancedServo,VelocityLimit,double limit)
//Actual velocity
CHDRGETINDEX(AdvancedServo,Velocity,double *velocity)
CHDRGETINDEX(AdvancedServo,VelocityMax,double *max)
CHDRGETINDEX(AdvancedServo,VelocityMin,double *min)
CHDREVENTINDEX(AdvancedServo,VelocityChange,double velocity)

CHDRGETINDEX(AdvancedServo,Position,double *position)
CHDRSETINDEX(AdvancedServo,Position,double position)
CHDRGETINDEX(AdvancedServo,PositionMax,double *max)
CHDRSETINDEX(AdvancedServo,PositionMax,double max)
CHDRGETINDEX(AdvancedServo,PositionMin,double *min)
CHDRSETINDEX(AdvancedServo,PositionMin,double min)
CHDREVENTINDEX(AdvancedServo,PositionChange,double position)

CHDRGETINDEX(AdvancedServo,Current,double *current)
CHDREVENTINDEX(AdvancedServo,CurrentChange,double current)

CHDRGETINDEX(AdvancedServo,SpeedRampingOn,int *rampingState)
CHDRSETINDEX(AdvancedServo,SpeedRampingOn,int rampingState)
CHDRGETINDEX(AdvancedServo,Engaged,int *engagedState)
CHDRSETINDEX(AdvancedServo,Engaged,int engagedState)
CHDRGETINDEX(AdvancedServo,Stopped,int *stoppedState)

#ifndef EXTERNALPROTO
#define ADVSERVO_MAXSERVOS 8

#define ADVSERVO_NORMAL_PACKET	0x00
#define ADVSERVO_MINMAX_PACKET	0x10

//Flags - need to match those defined in firmware
#define MOTOR_DONE_ADVSERVO	 		0x20
#define MOTOR_DISABLED_ADVSERVO		0x40
#define NO_RAMPING_FLAG_ADVSERVO	0x80

struct _CPhidgetAdvancedServo {
	CPhidget phid;

	int (CCONV *fptrPositionChange)(CPhidgetAdvancedServoHandle, void *, int, double);
	int (CCONV *fptrVelocityChange)(CPhidgetAdvancedServoHandle, void *, int, double);
	int (CCONV *fptrCurrentChange)(CPhidgetAdvancedServoHandle, void *, int, double);

	void *fptrPositionChangeptr;
	void *fptrVelocityChangeptr;
	void *fptrCurrentChangeptr;
	
	//data from the device
	double motorVelocityEcho[ADVSERVO_MAXSERVOS];
	double motorPositionEcho[ADVSERVO_MAXSERVOS];
	double motorSensedCurrent[ADVSERVO_MAXSERVOS];
	unsigned char motorSpeedRampingStateEcho[ADVSERVO_MAXSERVOS];
	unsigned char motorEngagedStateEcho[ADVSERVO_MAXSERVOS];
	unsigned char motorStoppedState[ADVSERVO_MAXSERVOS];
	int packetCounterEcho[ADVSERVO_MAXSERVOS];

	//data from the user
	double motorPosition[ADVSERVO_MAXSERVOS];
	double motorAcceleration[ADVSERVO_MAXSERVOS];
	double motorVelocity[ADVSERVO_MAXSERVOS];
	unsigned char motorSpeedRampingState[ADVSERVO_MAXSERVOS];
	unsigned char motorEngagedState[ADVSERVO_MAXSERVOS];
	int packetCounter[ADVSERVO_MAXSERVOS];
	double motorPositionMax[ADVSERVO_MAXSERVOS];
	double motorPositionMin[ADVSERVO_MAXSERVOS];
	
	double velocityMax, velocityMin;
	double accelerationMax, accelerationMin;
	double motorPositionMaxLimit, motorPositionMinLimit;

	unsigned char outputPacket[8];
	unsigned int outputPacketLen;
} typedef CPhidgetAdvancedServoInfo;
#endif

/** @} */

#endif
