#ifndef __CPHIDGETINTERFACEKIT
#define __CPHIDGETINTERFACEKIT
#include "cphidget.h"

/** \defgroup phidifkit Phidget InterfaceKit 
 * \ingroup phidgets
 * @{
 */

DPHANDLE(InterfaceKit)
CHDRSTANDARD(InterfaceKit)

CHDRGET(InterfaceKit, InputCount, int *count)
CHDRGETINDEX(InterfaceKit, InputState, int *inputState)
CHDREVENTINDEX(InterfaceKit, InputChange, int inputState)

CHDRGET(InterfaceKit, OutputCount, int *count)
CHDRGETINDEX(InterfaceKit, OutputState, int *outputState)
CHDRSETINDEX(InterfaceKit, OutputState, int outputState)
CHDREVENTINDEX(InterfaceKit, OutputChange, int outputState)

CHDRGET(InterfaceKit, SensorCount, int *count)
CHDRGETINDEX(InterfaceKit, SensorValue, int *sensorValue)
CHDRGETINDEX(InterfaceKit, SensorRawValue, int *sensorRawValue)
CHDREVENTINDEX(InterfaceKit, SensorChange, int sensorValue)
CHDRGETINDEX(InterfaceKit, SensorChangeTrigger, int *trigger)
CHDRSETINDEX(InterfaceKit, SensorChangeTrigger, int trigger)
CHDRGET(InterfaceKit, Ratiometric, int *ratiometric)
CHDRSET(InterfaceKit, Ratiometric, int ratiometric)

#ifndef REMOVE_DEPRECATED
DEP_CHDRGET("Deprecated - use CPhidgetInterfaceKit_getInputCount",InterfaceKit, NumInputs, int *)
DEP_CHDRGET("Deprecated - use CPhidgetInterfaceKit_getOutputCount",InterfaceKit, NumOutputs, int *)
DEP_CHDRGET("Deprecated - use CPhidgetInterfaceKit_getSensorCount",InterfaceKit, NumSensors, int *)
#endif

#ifndef EXTERNALPROTO
#define IFKIT_MAXINPUTS 32
#define IFKIT_MAXOUTPUTS 32
#define IFKIT_MAXSENSORS 8

#define IFKIT_MAXSENSORCHANGE 1000 //BL: Had to check for this, might as well use a define

struct _CPhidgetInterfaceKit {
	CPhidget phid;

	int (CCONV *fptrOutputChange)(CPhidgetInterfaceKitHandle, void *, int, int);
	int (CCONV *fptrInputChange)(CPhidgetInterfaceKitHandle, void *, int, int);
	int (CCONV *fptrSensorChange)(CPhidgetInterfaceKitHandle, void *, int, int);

	void *fptrOutputChangeptr;
	void *fptrInputChangeptr;
	void *fptrSensorChangeptr;

	unsigned char ratiometric;

	// output fields all protected by outputLock
	unsigned char outputStates[IFKIT_MAXOUTPUTS];
	unsigned char nextOutputStates[IFKIT_MAXOUTPUTS];
	unsigned char changedOutputs[IFKIT_MAXOUTPUTS];
	int lastChangedOutput;

	unsigned char outputEchoStates[IFKIT_MAXOUTPUTS];		// Values returned from the device
	unsigned char physicalState[IFKIT_MAXINPUTS];

	int sensorChangeTrigger[IFKIT_MAXSENSORS];
	int sensorRawValue[IFKIT_MAXSENSORS];
	int sensorValue[IFKIT_MAXSENSORS];
	int sensorLastValue[IFKIT_MAXSENSORS];

	unsigned char fullStateEcho;
} typedef CPhidgetInterfaceKitInfo;
#endif

/** @} */

#endif
