#ifndef __CPHIDGETSTEPPER
#define __CPHIDGETSTEPPER
#include "cphidget.h"

/** \defgroup phidstepper Phidget Stepper 
 * \ingroup phidgets
 * @{
 */

DPHANDLE(Stepper)
CHDRSTANDARD(Stepper)

CHDRGET(Stepper,InputCount,int *count)
CHDRGETINDEX(Stepper,InputState,int *inputState)
CHDREVENTINDEX(Stepper,InputChange,int inputState)

CHDRGET(Stepper,MotorCount,int *count)

CHDRGETINDEX(Stepper,Acceleration,double *acceleration)
CHDRSETINDEX(Stepper,Acceleration,double acceleration)
CHDRGETINDEX(Stepper,AccelerationMax,double *max)
CHDRGETINDEX(Stepper,AccelerationMin,double *min)

CHDRGETINDEX(Stepper,VelocityLimit,double *limit)
CHDRSETINDEX(Stepper,VelocityLimit,double limit)
CHDRGETINDEX(Stepper,Velocity,double *velocity)
CHDRGETINDEX(Stepper,VelocityMax,double *max)
CHDRGETINDEX(Stepper,VelocityMin,double *min)
CHDREVENTINDEX(Stepper,VelocityChange,double velocity)

CHDRGETINDEX(Stepper,TargetPosition,__int64 *position)
CHDRSETINDEX(Stepper,TargetPosition,__int64 position)
CHDRGETINDEX(Stepper,CurrentPosition,__int64 *positin)
CHDRSETINDEX(Stepper,CurrentPosition,__int64 position)
CHDRGETINDEX(Stepper,PositionMax,__int64 *max)
CHDRGETINDEX(Stepper,PositionMin,__int64 *min)
CHDREVENTINDEX(Stepper,PositionChange,__int64 position)

CHDRGETINDEX(Stepper,CurrentLimit,double *limit)
CHDRSETINDEX(Stepper,CurrentLimit,double limit)
CHDRGETINDEX(Stepper,Current,double *current)
CHDRGETINDEX(Stepper,CurrentMax,double *max)
CHDRGETINDEX(Stepper,CurrentMin,double *min)
CHDREVENTINDEX(Stepper,CurrentChange,double current)

CHDRGETINDEX(Stepper,Engaged,int *engagedState)
CHDRSETINDEX(Stepper,Engaged,int engagedState)
CHDRGETINDEX(Stepper,Stopped,int *stopped)

#ifndef EXTERNALPROTO

#define STEPPER_MAXSTEPPERS 8
#define STEPPER_MAXINPUTS 8

#define BIPOLAR_STEPPER_CURRENT_SENSE_GAIN 8.5
#define BIPOLAR_STEPPER_CURRENT_LIMIT_Rs 0.150

//flags - make sure these are in the upper 4 bits
#define MOTOR_DONE_STEPPER	 	0x10
#define MOTOR_DISABLED_STEPPER	0x20

//packet types - room for one more
#define STEPPER_POSITION_PACKET		0x00
#define STEPPER_VEL_ACCEL_PACKET	0x10
#define STEPPER_RESET_PACKET		0x20

struct _CPhidgetStepper{
	CPhidget phid;
          
	int (CCONV *fptrPositionChange)(CPhidgetStepperHandle, void *, int, __int64);        
	int (CCONV *fptrVelocityChange)(CPhidgetStepperHandle, void *, int, double);                   
	int (CCONV *fptrCurrentChange)(CPhidgetStepperHandle, void *, int, double);       
	int (CCONV *fptrInputChange)(CPhidgetStepperHandle, void *, int, int);           

	void *fptrPositionChangeptr;
	void *fptrInputChangeptr;
	void *fptrCurrentChangeptr;
	void *fptrVelocityChangeptr;

	//data from the device
	unsigned char inputState[STEPPER_MAXINPUTS];
	__int64 motorPositionEcho[STEPPER_MAXSTEPPERS];
	double motorSpeedEcho[STEPPER_MAXSTEPPERS];
	double motorSensedCurrent[STEPPER_MAXSTEPPERS];
	unsigned char motorEngagedStateEcho[STEPPER_MAXSTEPPERS];
	unsigned char motorStoppedState[STEPPER_MAXSTEPPERS];
	int packetCounterEcho[STEPPER_MAXSTEPPERS];

	//data from the user
	__int64 motorPosition[STEPPER_MAXSTEPPERS];
	__int64 motorPositionReset[STEPPER_MAXSTEPPERS];
	double motorAcceleration[STEPPER_MAXSTEPPERS];
	double motorSpeed[STEPPER_MAXSTEPPERS];
	double motorCurrentLimit[STEPPER_MAXSTEPPERS];
	unsigned char motorEngagedState[STEPPER_MAXSTEPPERS];
	int packetCounter[STEPPER_MAXSTEPPERS];
	
	double motorSpeedMax, motorSpeedMin;
	double accelerationMax, accelerationMin;
	__int64 motorPositionMax, motorPositionMin;
	double currentMax, currentMin;
	int microSteps;

	unsigned char outputPacket[8];
	unsigned int outputPacketLen;
} typedef CPhidgetStepperInfo;
#endif

/** @} */

#endif

