#ifndef __CPHIDGETTEMPERATURESENSOR
#define __CPHIDGETTEMPERATURESENSOR
#include "cphidget.h"

/** \defgroup phidtemp Phidget Temperature Sensor 
 * \ingroup phidgets
 * @{
 */

DPHANDLE(TemperatureSensor)
CHDRSTANDARD(TemperatureSensor)

typedef enum {
	PHIDGET_TEMPERATURE_SENSOR_K_TYPE = 1,
	PHIDGET_TEMPERATURE_SENSOR_J_TYPE,
	PHIDGET_TEMPERATURE_SENSOR_E_TYPE,
	PHIDGET_TEMPERATURE_SENSOR_T_TYPE
}  CPhidgetTemperatureSensor_ThermocoupleType;

CHDRGET(TemperatureSensor,TemperatureInputCount,int *count)
CHDRGETINDEX(TemperatureSensor,Temperature,double *temperature)
CHDRGETINDEX(TemperatureSensor,TemperatureMax,double *max)
CHDRGETINDEX(TemperatureSensor,TemperatureMin,double *min)
CHDREVENTINDEX(TemperatureSensor,TemperatureChange,double temperature)
CHDRGETINDEX(TemperatureSensor,TemperatureChangeTrigger,double *trigger)
CHDRSETINDEX(TemperatureSensor,TemperatureChangeTrigger,double trigger)

CHDRGETINDEX(TemperatureSensor,Potential,double *potential)
CHDRGETINDEX(TemperatureSensor,PotentialMax,double *max)
CHDRGETINDEX(TemperatureSensor,PotentialMin,double *min)

CHDRGET(TemperatureSensor,AmbientTemperature,double *ambient)
CHDRGET(TemperatureSensor,AmbientTemperatureMax,double *max)
CHDRGET(TemperatureSensor,AmbientTemperatureMin,double *min)

CHDRGETINDEX(TemperatureSensor, ThermocoupleType, CPhidgetTemperatureSensor_ThermocoupleType *type)
CHDRSETINDEX(TemperatureSensor, ThermocoupleType, CPhidgetTemperatureSensor_ThermocoupleType type)

#ifndef REMOVE_DEPRECATED
DEP_CHDRGET("Deprecated - use CPhidgetTemepratureSensor_getTemperatureInputCount",TemperatureSensor,NumTemperatureInputs,int *)
#endif

#ifndef EXTERNALPROTO
#define TEMPSENSOR_MAXSENSORS 8

#define GAIN 85.0
#define OFFSET_200 -6.5
#define OFFSET_300 ((200.0/237.0)*5.0)

struct _CPhidgetTemperatureSensor {
	CPhidget phid;

	int (CCONV *fptrTemperatureChange)(CPhidgetTemperatureSensorHandle, void *, int, double);           
	void *fptrTemperatureChangeptr;

	double AmbientTemperature;
	double Temperature[TEMPSENSOR_MAXSENSORS];
	double Potential[TEMPSENSOR_MAXSENSORS];

	double lastTrigger[TEMPSENSOR_MAXSENSORS];

	double TempChangeTrigger[TEMPSENSOR_MAXSENSORS];
	CPhidgetTemperatureSensor_ThermocoupleType ThermocoupleType[TEMPSENSOR_MAXSENSORS];

	double ambientTemperatureMax, ambientTemperatureMin;
	double temperatureMax[TEMPSENSOR_MAXSENSORS], temperatureMin[TEMPSENSOR_MAXSENSORS];
	double potentialMax, potentialMin;
} typedef CPhidgetTemperatureSensorInfo;
#endif

/** @} */

#endif
